@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2003-2008 Edgewall Software
+# Copyright (C) 2003-2009 Edgewall Software
 # Copyright (C) 2003-2007 Jonas Borgström <jonas@edgewall.com>
 # All rights reserved.
 #
@@ -29,6 +29,7 @@ from trac.core import Component, ComponentManager, implements, Interface, \
                       ExtensionPoint, TracError
 from trac.db import DatabaseManager
 from trac.util import get_pkginfo
+from trac.util.text import exception_to_unicode
 from trac.util.translation import _
 from trac.versioncontrol import RepositoryManager
 from trac.web.href import Href
@@ -120,6 +121,14 @@ class Environment(Component, ComponentManager):
     project_admin = Option('project', 'admin', '',
         """E-Mail address of the project's administrator.""")
 
+    project_admin_trac_url = Option('project', 'admin_trac_url', '.',
+        """Base URL of a Trac instance where errors in this Trac should be
+        reported.
+        
+        This can be an absolute or relative URL, or '.' to reference this
+        Trac instance. An empty value will disable the reporting buttons.
+        (''since 0.11.3'')""")
+
     project_footer = Option('project', 'footer',
                             'Visit the Trac open source project at<br />'
                             '<a href="http://trac.edgewall.org/">'
@@ -581,7 +590,8 @@ def open_environment(env_path=None, use_cache=False):
         try:
             needs_upgrade = env.needs_upgrade()
         except Exception, e: # e.g. no database connection
-            env.log.exception(e)
+            env.log.error("Exception caught while checking for upgrade: %s",
+                          exception_to_unicode(e, traceback=True))
         if needs_upgrade:
             raise TracError(_('The Trac Environment needs to be upgraded.\n\n'
                               'Run "trac-admin %(path)s upgrade"',
