@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2004-2008 Edgewall Software
+# Copyright (C) 2004-2009 Edgewall Software
 # Copyright (C) 2004-2005 Christopher Lenz <cmlenz@gmx.de>
 # Copyright (C) 2005-2007 Christian Boos <cboos@neuf.fr>
 # All rights reserved.
@@ -30,7 +30,7 @@ from trac.mimeview.api import Mimeview, IContentConverter, Context
 from trac.resource import Resource
 from trac.ticket.api import TicketSystem
 from trac.util import Ranges
-from trac.util.compat import groupby
+from trac.util.compat import groupby, set
 from trac.util.datefmt import to_timestamp, utc
 from trac.util.presentation import Paginator
 from trac.util.text import shorten_line
@@ -130,21 +130,21 @@ class Query(object):
             if len(filter_) != 2:
                 raise QuerySyntaxError(_('Query filter requires field and ' 
                                          'constraints separated by a "="'))
-            field,values = filter_
-            if not field:
-                raise QuerySyntaxError(_('Query filter requires field name'))
-            # from last char of `field`, get the mode of comparison
-            mode, neg = '', ''
-            if field[-1] in ('~', '^', '$') \
+            field, values = filter_
+            # from last chars of `field`, get the mode of comparison
+            mode = ''
+            if field and field[-1] in ('~', '^', '$') \
                                 and not field in cls.substitutions:
                 mode = field[-1]
                 field = field[:-1]
-            if field[-1] == '!':
-                neg = '!'
+            if field and field[-1] == '!':
+                mode = '!' + mode
                 field = field[:-1]
+            if not field:
+                raise QuerySyntaxError(_('Query filter requires field name'))
             processed_values = []
             for val in values.split('|'):
-                val = neg + mode + val # add mode of comparison
+                val = mode + val # add mode of comparison
                 processed_values.append(val)
             try:
                 if isinstance(field, unicode):
@@ -152,13 +152,13 @@ class Query(object):
                 if field in kw_strs:
                     kw[field] = processed_values[0]
                 elif field in kw_arys:
-                    kw[field] = processed_values
+                    kw.setdefault(field, []).extend(processed_values)
                 elif field in kw_bools:
                     kw[field] = True
                 elif field == 'col':
                     cols.extend(processed_values)
                 else:
-                    constraints[field] = processed_values
+                    constraints.setdefault(field, []).extend(processed_values)
             except UnicodeError:
                 pass # field must be a str, see `get_href()`
         report = constraints.pop('report', None)
@@ -188,21 +188,6 @@ class Query(object):
         # TODO: fix after adding time/changetime to the api.py
         cols += ['time', 'changetime']
 
-        # Semi-intelligently remove columns that are restricted to a single
-        # value by a query constraint.
-        for col in [k for k in self.constraints.keys()
-                    if k != 'id' and k in cols]:
-            constraint = self.constraints[col]
-            if len(constraint) == 1 and constraint[0] \
-                    and not constraint[0][0] in ('!', '~', '^', '$'):
-                if col in cols:
-                    cols.remove(col)
-            if col == 'status' and not 'closed' in constraint \
-                    and 'resolution' in cols:
-                cols.remove('resolution')
-        if self.group in cols:
-            cols.remove(self.group)
-
         def sort_columns(col1, col2):
             constrained_fields = self.constraints.keys()
             if 'id' in (col1, col2):
@@ -219,9 +204,24 @@ class Query(object):
         return cols
 
     def get_default_columns(self):
-        all_cols = self.get_all_columns()
+        cols = self.get_all_columns()
+        
+        # Semi-intelligently remove columns that are restricted to a single
+        # value by a query constraint.
+        for col in [k for k in self.constraints.keys()
+                    if k != 'id' and k in cols]:
+            constraint = self.constraints[col]
+            if len(constraint) == 1 and constraint[0] \
+                    and not constraint[0][0] in '!~^$' and col in cols:
+                cols.remove(col)
+            if col == 'status' and not 'closed' in constraint \
+                    and 'resolution' in cols:
+                cols.remove('resolution')
+        if self.group in cols:
+            cols.remove(self.group)
+
         # Only display the first seven columns by default
-        cols = all_cols[:7]
+        cols = cols[:7]
         # Make sure the column we order by is visible, if it isn't also
         # the column we group by
         if not self.order in cols and not self.order == self.group:
@@ -304,7 +304,7 @@ class Query(object):
                 elif field and field['type'] == 'checkbox':
                     try:
                         val = bool(int(val))
-                    except TypeError, ValueError:
+                    except (TypeError, ValueError):
                         val = False
                 result[name] = val
             results.append(result)
@@ -574,6 +574,8 @@ class Query(object):
 
         cols = self.get_columns()
         labels = dict([(f['name'], f['label']) for f in self.fields])
+        wikify = set([f['name'] for f in self.fields 
+                      if f['type'] == 'text' and f.get('format') == 'wiki'])
 
         # TODO: remove after adding time/changetime to the api.py
         labels['changetime'] = _('Modified')
@@ -581,12 +583,16 @@ class Query(object):
 
         headers = [{
             'name': col, 'label': labels.get(col, _('Ticket')),
+            'wikify': col in wikify,
             'href': self.get_href(context.href, order=col,
                                   desc=(col == self.order and not self.desc))
         } for col in cols]
 
         fields = {}
         for field in self.fields:
+            if field['name'] == 'owner' and field['type'] == 'select':
+                # Make $USER work when restrict_owner = true
+                field['options'].insert(0, '$USER')
             field_data = {}
             field_data.update(field)
             del field_data['name']
@@ -942,7 +948,6 @@ class QueryModule(Component):
     def export_csv(self, req, query, sep=',', mimetype='text/plain'):
         content = StringIO()
         cols = query.get_columns()
-        writer = csv.writer(content, delimiter=sep)
         writer = csv.writer(content, delimiter=sep, quoting=csv.QUOTE_MINIMAL)
         writer.writerow([unicode(c).encode('utf-8') for c in cols])
 
@@ -1105,6 +1110,7 @@ class TicketQueryMacro(WikiMacroBase):
                          class_=ticket['status'],
                          href=req.href.ticket(int(ticket['id'])),
                          title=shorten_line(ticket['summary']))
+
         def ticket_groups():
             groups = []
             for v, g in groupby(tickets, lambda t: t[query.group]):
@@ -1113,8 +1119,9 @@ class TicketQueryMacro(WikiMacroBase):
                 q.group = q.groupdesc = None
                 order = q.order
                 q.order = None
-                title = "%s %s tickets matching %s" % (v, query.group,
-                                                       q.to_string())
+                title = _("%(groupvalue)s %(groupname)s tickets matching "
+                          "%(query)s", groupvalue=v, groupname=query.group,
+                          query=q.to_string())
                 # produce the href for the query corresponding to the group
                 q.constraints[str(query.group)] = v
                 q.order = order
@@ -1124,9 +1131,9 @@ class TicketQueryMacro(WikiMacroBase):
 
         if format == 'compact':
             if query.group:
-                groups = [tag.a('#%s' % ','.join([str(t['id'])
-                                                  for t in g]),
-                                href=href, class_='query', title=title)
+                groups = [(v, ' ', 
+                           tag.a('#%s' % ','.join([str(t['id']) for t in g]),
+                                 href=href, class_='query', title=title))
                           for v, g, href, title in ticket_groups()]
                 return tag(groups[0], [(', ', g) for g in groups[1:]])
             else:
