@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2003-2008 Edgewall Software
+# Copyright (C) 2003-2009 Edgewall Software
 # Copyright (C) 2003-2005 Jonas Borgström <jonas@edgewall.com>
 # All rights reserved.
 #
@@ -34,13 +34,14 @@ from trac.resource import Resource, get_resource_url, \
 from trac.search import ISearchSource, search_to_sql, shorten_result
 from trac.ticket.api import TicketSystem, ITicketManipulator, \
                             ITicketActionController
-from trac.ticket.model import Milestone, Ticket
+from trac.ticket.model import Milestone, Ticket, group_milestones
 from trac.ticket.notification import TicketNotifyEmail
 from trac.timeline.api import ITimelineEventProvider
-from trac.util import get_reporter_id, partition
+from trac.util import get_reporter_id
 from trac.util.compat import any
 from trac.util.datefmt import to_timestamp, utc
-from trac.util.text import CRLF, shorten_line, obfuscate_email_address
+from trac.util.text import CRLF, shorten_line, obfuscate_email_address, \
+                           exception_to_unicode
 from trac.util.presentation import separated
 from trac.util.translation import _
 from trac.versioncontrol.diff import get_diff_options, diff_blocks
@@ -48,7 +49,7 @@ from trac.web import IRequestHandler
 from trac.web.chrome import add_link, add_script, add_stylesheet, \
                             add_warning, add_ctxtnav, prevnext_nav, Chrome, \
                             INavigationContributor, ITemplateProvider
-from trac.wiki.formatter import format_to
+from trac.wiki.formatter import format_to, format_to_html, format_to_oneliner
 
 class InvalidTicket(TracError):
     """Exception raised when a ticket fails validation."""
@@ -100,8 +101,8 @@ class TicketModule(Component):
         (''since 0.11'').""")
 
     timeline_details = BoolOption('timeline', 'ticket_show_details', 'false',
-        """Enable the display of all ticket changes in the timeline
-        (''since 0.9'').""")
+        """Enable the display of all ticket changes in the timeline, not only
+        open / close operations (''since 0.9'').""")
 
     max_description_size = IntOption('ticket', 'max_description_size', 262144,
         """Don't accept tickets with a too big description.
@@ -235,9 +236,9 @@ class TicketModule(Component):
 
     def get_timeline_filters(self, req):
         if 'TICKET_VIEW' in req.perm:
-            yield ('ticket', _('Ticket changes'))
+            yield ('ticket', _('Opened and closed tickets'))
             if self.timeline_details:
-                yield ('ticket_details', _('Ticket details'), False)
+                yield ('ticket_details', _('Ticket updates'), False)
 
     def get_timeline_events(self, req, start, stop, filters):
         ts_start = to_timestamp(start)
@@ -560,15 +561,23 @@ class TicketModule(Component):
         if global_sequence:
             db = self.env.get_db_cnx()
             cursor = db.cursor()
-            cursor.execute("SELECT max(id) FROM ticket")
-            for max_id, in cursor:
+            cursor.execute("SELECT min(id), max(id) FROM ticket")
+            for (min_id, max_id) in cursor:
+                min_id = int(min_id)
                 max_id = int(max_id)
-                if ticket.id > 1:
-                    add_ticket_link('first', 1)
-                    add_ticket_link('prev', ticket.id - 1)
+                if min_id < ticket.id:
+                    add_ticket_link('first', min_id)
+                    cursor.execute("SELECT max(id) FROM ticket WHERE id < %s",
+                                   (ticket.id,))
+                    for (prev_id,) in cursor:
+                        add_ticket_link('prev', int(prev_id))
                 if ticket.id < max_id:
-                    add_ticket_link('next', ticket.id + 1)
                     add_ticket_link('last', max_id)
+                    cursor.execute("SELECT min(id) FROM ticket WHERE %s < id",
+                                   (ticket.id,))
+                    for (next_id,) in cursor:
+                        add_ticket_link('next', int(next_id))
+                break
 
         add_stylesheet(req, 'common/css/ticket.css')
 
@@ -977,8 +986,8 @@ class TicketModule(Component):
             tn = TicketNotifyEmail(self.env)
             tn.notify(ticket, newticket=True)
         except Exception, e:
-            self.log.exception("Failure sending notification on creation of "
-                               "ticket #%s: %s" % (ticket.id, e))
+            self.log.error("Failure sending notification on creation of "
+                    "ticket #%s: %s", ticket.id, exception_to_unicode(e))
 
         # Redirect the user to the newly created ticket or add attachment
         if 'attachment' in req.args:
@@ -1007,8 +1016,8 @@ class TicketModule(Component):
                 tn = TicketNotifyEmail(self.env)
                 tn.notify(ticket, newticket=False, modtime=now)
             except Exception, e:
-                self.log.exception("Failure sending notification on change to "
-                                   "ticket #%s: %s" % (ticket.id, e))
+                self.log.error("Failure sending notification on change to "
+                        "ticket #%s: %s", ticket.id, exception_to_unicode(e))
 
         # After saving the changes, apply the side-effects.
         for controller in controllers:
@@ -1086,26 +1095,16 @@ class TicketModule(Component):
                         field['skip'] = False
                         owner_field = field
             elif name == 'milestone':
-                milestones = [(opt, Milestone(self.env, opt))
+                milestones = [Milestone(self.env, opt)
                               for opt in field['options']]
-                milestones = [(opt, m) for opt, m in milestones
+                milestones = [m for m in milestones
                               if 'MILESTONE_VIEW' in req.perm(m.resource)]
-                def category(m):
-                    return m.is_completed and 1 or m.due and 2 or 3
-                open_due_milestones, open_not_due_milestones, \
-                    closed_milestones = partition([(opt, category(m))
-                        for opt, m in milestones], (2, 3, 1))
+                groups = group_milestones(milestones, ticket.exists 
+                    and 'TICKET_ADMIN' in req.perm(ticket.resource))
                 field['options'] = []
                 field['optgroups'] = [
-                    {'label': _('Open (by due date)'), 
-                        'options': open_due_milestones},
-                    {'label': _('Open (no due date)'), 
-                        'options': open_not_due_milestones},
-                ]
-                if ticket.exists and \
-                       'TICKET_ADMIN' in req.perm(ticket.resource):
-                    field['optgroups'].append(
-                        {'label': _('Closed'), 'options': closed_milestones})
+                    {'label': label, 'options': [m.name for m in milestones]}
+                    for (label, milestones) in groups]
                 milestone = Resource('milestone', ticket[name])
                 field['rendered'] = render_resource_link(self.env, context,
                                                          milestone, 'compact')
@@ -1141,7 +1140,16 @@ class TicketModule(Component):
                 value = ticket.values.get(name)
                 if value in ('1', '0'):
                     field['rendered'] = value == '1' and _('yes') or _('no')
-                  
+            elif type_ == 'text':
+                if field.get('format') == 'wiki':
+                    field['rendered'] = format_to_oneliner(self.env, context,
+                                                           ticket[name])
+            elif type_ == 'textarea':
+                if field.get('format') == 'wiki':
+                    field['rendered'] = \
+                        format_to_html(self.env, context, ticket[name],
+                                escape_newlines=self.must_preserve_newlines)
+            
             # ensure sane defaults
             field.setdefault('optional', False)
             field.setdefault('options', [])
